;;
;; $Id: query.scm,v 1.11 2005/08/02 19:23:13 ben Exp $
;;
;; This module contains the collection of partial queries by the 
;; SPS application.
;;
;; The goal is to have all the queries stored in this file and used
;; througout the app
;;

(require-library 'com/amazingmedia/db/partial-query)
(require-library 'com/amazingmedia/util/context)
(require-library 'com/amazingmedia/types/shipping-address)
(require-library 'com/amazingmedia/types/credit-card)
(require-library 'sisc/libs/srfi/srfi-19)

(module com/amazingmedia/sps/db/query
    (all-queries)

  (import com/amazingmedia/db/partial-query)
  (import com/amazingmedia/util/context)
  (import com/amazingmedia/types/shipping-address)
  (import com/amazingmedia/types/credit-card)
  (import srfi-19)

  ;;
  ;; This function will return a context loaded with all the
  ;; queries in the system
  ;;
  ;; returns a context full of queries
  ;;
  (define (all-queries)
    (define-syntax q
      (syntax-rules ()
        ((_ arg ...)
         (make-partial-query arg ...))))

    (make-context
     'immutable
     `((search-by-product-name 
        .
        ,(q ("SELECT productId, name, description, image"
             "FROM Product"
             "WHERE"
             "  lower(name) LIKE '%' || ? || '%'")
            '(productId name description image)))

       (search
        .
        ,(q ("SELECT itemId, productId, name, description, image"
             "FROM search(?)")
            '(itemId productId name description image)))

       (find-products-by-category-name
        .
        ,(q ("SELECT p.productId AS productId, p.name FROM"
             "Product p, Category c"
             "WHERE p.categoryId = c.categoryId"
             "  AND lower(c.name) = lower(trim(?))")
            '(productId name)))

       (find-items-by-product-id
        .
        ,(q ("SELECT i.itemId, item_name(i.itemId) AS itemName, "
                    "i.listPrice, q.quantity"
                    "FROM   Product p, Item i, Inventory q"
                    "WHERE i.productId = p.productId"
                    "  AND lower(p.productId) = trim(lower(?))"
                    "  AND q.itemId = i.itemId")
            '(itemId itemName listPrice quantity)))

       (lookup-item-by-id
        .
        ,(q ("SELECT item_name(i.itemId) AS itemName,"
             "       p.name AS productName, p.productId AS productId,"
             "       i.description, i.listPrice, i.image, i.itemId, "
             "       q.quantity"
             "FROM Product p, Item i, Inventory q"
             "WHERE i.itemId    = ? AND"
             "      p.productId = i.productId AND"
             "      i.itemId    = q.itemId")
            '(itemName productName productId description
                       listPrice image itemId quantity)))

       (lookup-product-by-id
        .
        ,(q ("SELECT name, initcap(categoryId) AS categoryId "
             "FROM Product "
             "WHERE productId = ?")
            '(name categoryId)))

       (find-order-summary
        .
        ,(q ("SELECT"
             "  orderId, shipTo, total, purchaseDate "
             "FROM order_summary(?)")
            '(orderId shipTo total purchaseDate)))

       (find-order-details
        .
        ,(q ("SELECT"
             "  lineNum, itemId, item_name(itemId) AS itemName,"
             "  quantity, unitCost"
             "FROM OrderLineItem"
             "WHERE"
             "  orderId = ?")
            '(lineNum itemId itemName quantity unitCost)))

       (check-login
        .
        ,(q ("SELECT value, reason FROM check_login(?, ?)")
            '(value reason)))

       (username-exists?
        .
        ,(q ("SELECT user_exists(?) as exists") '(exists)))

       (register
        .
        ,(q ("SELECT register(?,?) as userId") '(userId)))

       (purchase
        .
        ,(q ("SELECT purchase("
             "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?"
             ") AS orderId")
            '(orderId)
            (lambda (username cc bill-addr ship-addr)
              (list 
               username
               (credit-card-number cc)
               (credit-card-type cc)
               (date-month (credit-card-expires cc))
               (date-year (credit-card-expires cc))
               (shipping-address-first-name bill-addr)
               (shipping-address-last-name bill-addr)
               (shipping-address-street1 bill-addr)
               (shipping-address-street2 bill-addr)
               (shipping-address-city bill-addr)
               (shipping-address-state bill-addr)
               (shipping-address-zip bill-addr)
               (shipping-address-first-name ship-addr)
               (shipping-address-last-name ship-addr)
               (shipping-address-street1 ship-addr)
               (shipping-address-street2 ship-addr)
               (shipping-address-city ship-addr)
               (shipping-address-state ship-addr)
               (shipping-address-zip ship-addr)))))

       (add-line-item
        .
        ,(q ("SELECT add_line_item(?, ?, ?, ?) AS lineItemId")
            '(lineItemId)))

       
       )))

  )