;;
;; $Id: cart.scm,v 1.12 2005/07/25 19:11:22 ben Exp $
;;
;; Scheme code to render and add stuff to carts
;;

(require-library 'sisc/libs/srfi/srfi-16)

(require-library 'com/amazingmedia/sps/web/button)
(require-library 'com/amazingmedia/util/context)
(require-library 'com/amazingmedia/web/item)
(require-library 'com/amazingmedia/sps/web/context-helper)

(module com/amazingmedia/sps/ui/cart 
    (cart/view
     cart/add
     cart/update
     cart/remove
     cart/contents)

  (import com/amazingmedia/sps/web/button)
  (import com/amazingmedia/sps/web/context-helper)
  (import com/amazingmedia/web/item)
  (import com/amazingmedia/util/context)
  (import srfi-16)

  ;;
  ;; return an xml representation of our cart
  ;;
  (define cart/contents 
    (case-lambda 
     ((ctx) (cart/contents ctx #f))
     ((ctx hl-item-id)
      `(cart
        ,@(context/map 'cart
                       (lambda (i)
                         (let-query ctx lookup-item-by-id
                                    ((item-id i))
                                    (item-name 
                                     product-name product-id
                                     description list-price
                                     image item-id)
                         `(item (id ,item-id)
                                (name ,item-name)
                                (selected ,(equal? hl-item-id item-id))
                                (image ,image)
                                (price ,(item-price i))
                                (quantity ,(item-quantity i)))))
                       ctx)))))
    
  (define cart/view
    (case-lambda
     ((ctx)
      (cart/view ctx #f))
     ((ctx message)
      (cart/view ctx message #f))
     ((ctx message hl-item-id)
      (render (add-error ctx message) (cart/contents ctx hl-item-id)))))

  (define (cart/add ctx item-id price quantity)
    (let ((existing-item (context/get `(cart ,item-id) ctx)))
      (cart/check-inventory ctx item-id 
                            (if existing-item
                                (+ (item-quantity existing-item) quantity)
                                quantity)
                            (lambda ()
                              (cart/view 
                               (context/add `(cart ,item-id)
                                            (make-item item-id price quantity)
                                            ctx)
                               #f item-id)))))

  (define (cart/update ctx item-id requested-quantity)
    (cart/check-inventory ctx item-id requested-quantity
                          (lambda ()
                            (cart/view 
                             (let ((orig (context/get `(cart ,item-id) ctx)))
                               (context/put `(cart ,item-id)
                                            (update-item-quantity orig requested-quantity)
                                            ctx))
                             #f item-id))))




  (define (cart/check-inventory ctx item-id requested-quantity ok-k)
    (let-query ctx lookup-item-by-id
               (item-id)
               (item-name product-name product-id description 
                          list-price image item-id actual-quantity)
      (cond ((> requested-quantity actual-quantity)
             (cart/view ctx
                        (format (string-append "Oops: requested ~a, "
                                               "but only ~a left in inventory")
                                requested-quantity actual-quantity)
                        item-id))
            (else
             (ok-k)))))

  (define (cart/remove ctx item-id)
    (cart/view (context/remove `(cart ,item-id) ctx)))

  )