;;
;; $Id: checkout.scm,v 1.8 2005/08/09 13:15:51 ben Exp $
;;
;; Functions to support checking out of the store.
;;

(require-library 'com/amazingmedia/sps/web/context-helper)
(require-library 'com/amazingmedia/sps/web/button)
(require-library 'com/amazingmedia/sps/web/default)
(require-library 'com/amazingmedia/types/shipping-address)
(require-library 'com/amazingmedia/types/credit-card)
(require-library 'com/amazingmedia/web/item)
(require-library 'sisc/libs/srfi/srfi-42)

(module com/amazingmedia/sps/ui/checkout
    (checkout/start)

  (import com/amazingmedia/types/credit-card)
  (import com/amazingmedia/types/shipping-address)
  (import com/amazingmedia/web/item)
  (import com/amazingmedia/sps/web/context-helper)
  (import com/amazingmedia/sps/web/button)
  (import com/amazingmedia/sps/web/default)
  (import com/amazingmedia/sps/ui/cart)
  (import srfi-42)

  ;;
  ;; This procedure starts the checkout process.  It's the 
  ;; entry point into the whole checkout process
  ;;
  (define (checkout/start ctx)
    (if (logged-in? ctx)
        (prompt-for-billing-data ctx
                                 default/cc default/bill-to
                                 default/phone #f)
        (apply-nav ctx 'login default/username default/password
                   (lambda (ctx)
                     (prompt-for-billing-data ctx default/cc 
                                              default/bill-to 
                                              default/phone #f)))))
  
  (define (prompt-for-billing-data ctx 
                                   cc billing-addr phone 
                                   ship-bill-same?)
    (render 
     ctx
     `(basic-page
       (title "Enter Billing Info")
       ,(use-form ctx 'billing
                  (lambda (ctx cc billing-addr phone ship-bill-same?)
                    (cond (ship-bill-same?
                           (confirm-entered-info ctx 
                                                 cc
                                                 billing-addr
                                                 billing-addr))
                            (else
                             (prompt-for-shipping-data ctx 
                                                       cc
                                                       billing-addr
                                                       default/ship-to 
                                                       default/phone))))
                  (lambda (ctx cc billing-addr phone ship-bill-same?)
                    (prompt-for-billing-data
                     ctx cc billing-addr phone ship-bill-same?))
                  cc billing-addr phone ship-bill-same?))))


  (define (prompt-for-shipping-data ctx credit-card billing-address ship-address phone)
    (render
     ctx
     `(basic-page
       (title "Enter Shipping Info")
       ,(use-form ctx 'shipping
                  (lambda (ctx shipping-address phone)
                    (confirm-entered-info ctx credit-card billing-address
                                          shipping-address))
                  (lambda (ctx shipping-address phone)
                    (prompt-for-shipping-data 
                     ctx credit-card billing-address shipping-address phone))
                  ship-address phone))))

  (define (confirm-entered-info ctx credit-card billing-address shipping-address)
    (render 
     ctx
     `(main-column-layout
       (title "Confirm Order")
       (controls ,(button/generic ctx "Check Out"
                                  (lambda (ctx)
                                    (do-charge ctx
                                               credit-card
                                               billing-address
                                               shipping-address)))
                 ,(button/view-cart ctx "Update Your Order"))
       (section 
        "Order Contents"
        ,(cart/contents ctx))
       (section
        "Credit Card Info"
        (info-block ,credit-card))
       (section
        "Billing Address"
        (info-block ,billing-address))
       (section
        "Shipping Address"
        (info-block ,shipping-address)))))

  (define (do-charge ctx cc bill-addr ship-addr)
    (with-conn conn
      (let-query ctx purchase (connection conn)
                 ((logged-in-as ctx) cc bill-addr ship-addr)
                 (order-id)
        (map-cart ctx
                  (lambda (item)
                    (let-query ctx add-line-item 
                               (connection conn)
                               (order-id 
                                (item-id item)
                                (item-quantity item)
                                (item-price item))
                      (lineNum)
                      'done)))
        (render-final
         (empty-cart ctx)
         `(basic-page
           (title "Order Successful")
           (div
            (p "Congrats, your order has been successfully purchased.")
            (p "Your order ID is: " ,order-id)
            (p ,(button/landing (empty-cart ctx) "Shop some more")
               " "
               ,(button/view-account (empty-cart ctx)))))))))
      
  )