;;
;; $Id: button.scm,v 1.25 2005/08/08 18:44:24 ben Exp $
;;
;; Scheme code to to render buttons on the site. This is just a collection
;; of parameterized buttons
;;

(require-library 'sisc/libs/srfi/srfi-16)
(require-library 'sisc/libs/srfi/srfi-26)
(require-library 'com/amazingmedia/util/context)
(require-library 'com/amazingmedia/sps/web/context-helper)
(require-library 'com/amazingmedia/sps/web/default)

(require-library 'com/amazingmedia/web/messages)


(module com/amazingmedia/sps/web/button
    (button/add-to-cart
     button/remove-from-cart
     button/update-cart
     button/view-cart
     button/sign-in
     button/logout
     button/help
     button/view-product
     button/view-category
     button/view-item
     button/landing
     button/search
     button/checkout
     button/continue
     button/view-account
     button/generic)

  (import srfi-26)
  (import srfi-16)
  (import com/amazingmedia/util/context)
  (import com/amazingmedia/web/messages)
  (import com/amazingmedia/sps/web/context-helper)
  (import com/amazingmedia/sps/web/default)

  (define (not-implemented what)
    (lambda (ctx)
      (error (format "~a not yet implemented" what))))

  ;;
  ;; Render a given button, assuming we have some known
  ;; text and that `link' is a list of components that
  ;; need to be turned into strings and appended together
  ;;
  (define (render-button ctx text action)
    `(a (@ (href-p ,(lambda (req) 
                      (set! req #f) 
                      (action (reset-context ctx)))))
        ,text))


   (define (button/add-to-cart ctx item-id price quantity)
     (render-button ctx "Add to Cart"
                    (lambda (ctx)
                      (apply-nav ctx 'add-to-cart item-id price quantity))))

   (define button/view-cart 
     (case-lambda
      ((ctx) (button/view-cart ctx "View Cart"))
      ((ctx text)
       (render-button ctx text
                      (lambda (ctx)
                        (apply-nav ctx 'view-cart))))))

   (define button/view-account 
     (case-lambda
      ((ctx) (button/view-account ctx "View Account"))
      ((ctx text)
       (render-button ctx text
                      (lambda (ctx)
                        (apply-nav ctx 'view-account))))))

   (define (button/help ctx)
     (render-button ctx "Help" 
                    (not-implemented "help")))

   (define (button/sign-in ctx)
     (render-button ctx "Sign In" 
                    (lambda (ctx)
                      (apply-nav ctx 'login
                                 default/username
                                 default/password
                                 (lambda (ctx)
                                   (apply-nav ctx 'view-account))))))

   (define (button/logout ctx)
     (render-button ctx "Sign Out"
                    (lambda (ctx)
                      (apply-nav ctx 'logout))))

   (define (button/view-product ctx prefix product-id name)
     (render-button ctx (string-append prefix name)
                    (lambda (ctx)
                      (apply-nav
                       (add-history ctx name
                                    (lambda (ctx)
                                      (apply-nav ctx 'view-product product-id)))
                       'view-product
                       product-id))))


  (define (button/view-category ctx prefix name)
    (render-button ctx (string-append prefix name)
                    (lambda (ctx)
                      (apply-nav
                       (add-history ctx name
                                    (lambda (ctx)
                                      (apply-nav ctx 'view-category name)))
                       'view-category
                       name))))


   (define (button/view-item ctx item-id name)
     (render-button ctx name
                    (lambda (ctx)
                      (apply-nav 
                       (add-history ctx name
                                    (lambda (ctx)
                                      (apply-nav ctx 'view-item item-id)))
                       'view-item
                       item-id))))

   (define button/landing
     (case-lambda
      ((ctx) (button/landing ctx "Home"))
      ((ctx label)
       (render-button ctx label
                      (lambda (ctx)
                        (apply-nav ctx 'landing))))))

   (define (button/search ctx)
     (use-form 
      ctx 'search
      (lambda (ctx query)
        (let ((name  (string-append "Search: " query)))
          (apply-nav
           (add-history ctx name
                        (lambda (ctx)
                          (apply-nav ctx 'search query)))
           'search
           query)))
      (lambda (ctx query)
        (apply-nav ctx 'search "-- No Search Provided --"))
      ""))

   (define (button/remove-from-cart ctx item-id)
     (render-button ctx "Remove"
                    (lambda (ctx)
                      (apply-nav ctx 'remove-from-cart item-id))))

   (define (button/update-cart ctx item-id quantity)
     (use-form ctx 'update-cart
               (lambda (ctx quantity)
                 (apply-nav ctx 'update-cart item-id quantity))
               (lambda (ctx quantity)
                 (apply-nav ctx 'view-cart))
               quantity))

   (define (button/checkout ctx)
     (render-button ctx "Proceed to Checkout"
                    (lambda (ctx)
                      (apply-nav ctx 'checkout))))

   (define (button/continue ctx thunk)
     (render-button ctx "Continue" thunk))

   (define (button/generic ctx text thunk)
     (render-button ctx text thunk))
   )