;;
;; $Id: decoration.scm,v 1.11 2005/08/08 18:46:14 ben Exp $
;;
;; Scheme code to take a fragment of HTML and decorate it - adding
;; the headers, footers, etc.
;;
;; Technically, this could be part of layout.scm, but to make layout.scm
;; easier to digest, we put the decoration code in here.
;;

(require-library 'com/amazingmedia/sps/web/button)
(require-library 'com/amazingmedia/sps/web/context-helper)
(require-library 'com/amazingmedia/util/context)
(require-library 'com/amazingmedia/web/history)
(require-library 'com/amazingmedia/web/direct-link)
(require-library 'sisc/libs/srfi/srfi-13)

(module com/amazingmedia/sps/web/decoration
    (decorate-body)

  (import com/amazingmedia/sps/web/button)
  (import com/amazingmedia/sps/web/context-helper)
  (import com/amazingmedia/util/context)
  (import com/amazingmedia/web/messages)
  (import com/amazingmedia/web/history)
  (import com/amazingmedia/web/direct-link)
  (import srfi-13)

  ;;
  ;; This function will be used to take in formatted HTML and add
  ;; our headers, footers, etc. to it.
  ;;
  ;; body -- the main body to decorate
  ;; return a well layed out page
  ;;
  (define (decorate-body ctx body)
    (define (script . lines)
      `(script (@ (language "javascript"))
               (*COMMENT* 
                ,(string-append "\n"
                                (string-join lines "\n")
                                "\n//"))))
                          
               
    (define (header)
      `((tr
         (@ (class "header"))
         (td ,(button/landing ctx '(img (@ (src-c "/images/logo-topbar.gif") (border 0)))))
         (td ,(button/search ctx))       
         (td (@ (align "right"))
             ,(button/view-cart ctx) " | "
             ,(if (logged-in? ctx) 
                  (string-append "Logged in as: " 
                                 (logged-in-as ctx))
                  (button/sign-in ctx)) 
             ,@(if (logged-in? ctx)
                   (list " | " (button/view-account ctx)
                         " | " (button/logout ctx))
                   '())))
        ,(if (has-direct-link? ctx)
             `(tr (td (@ (id "directLink")
                         (colspan 3))
                      ,(render-direct-link (get-direct-link ctx))))
             "")
        ,(if (ui-page-show-category-header)
             `(tr (td (@ (class "categoryList") (colspan 3) (align "center"))
                      ,(button/view-category ctx "" "Fish") " | "
                      ,(button/view-category ctx "" "Dogs") " | "
                      ,(button/view-category ctx "" "Reptiles") " | "
                      ,(button/view-category ctx "" "Cats") " | "
                      ,(button/view-category ctx "" "Birds")))
             "")))
    (define (history)
      (if (has-history? ctx)
          `(div (@ (class "history"))
                (table (@ (cellpadding 0) (cellspacing 0) (border 0))
                       (tr (th "History"))
                       ,@(map-history ctx
                                      (lambda (h)
                                        `(tr (td (a (@ (href-p ,(lambda (req)
                                                                  (set! req #f)
                                                                  ((history-proc h) ctx)))))
                                                 ,(history-label h)))))
                       (tr (td (@ (class "control"))
                               (a (@ (href-p ,(lambda (req)
                                                (set! req #f)
                                                (apply-nav (clear-history 
                                                            b(reset-context ctx))
                                                           'landing))))
                                  "Clear History")))))
          ""))

    (define (footer)
      `((tr (@ (class "footer"))
            (td (@ (colspan 3)
                   (align "center"))
                (p "Powered By " (a (@ (href "http://siscweb.sourceforge.net")) "SISCweb"))))))

    (define (messages)
      (if (errors? ctx)
          `(tr (td (@ (class "pageMessage"))
                   (ul ,@(map (cut list 'li <>) 
                              (get-errors ctx)))))
          ""))

    
    (define (do-body body)
      `(tr (@ (height 480) (valign "top"))
           (td (@ (class "body")
                  (style ,(string-append "padding: " (ui-page-padding)))
                  (colspan 3))
               ,body)))
    `(html
      (head
       (title ,(ui-page-title))
       (link  (@ (href-c "/css/layout.css")
                 (rel "stylesheet")
                 (type "text/css"))))                       
      (body 
       (@ (class "page")
          (leftmargin 0) (topmargin 0))
       (script (@ (language "javascript")
                  (src-c "/js/directLink.js")) "1;")
       ,(history)
       (table 
        (@ (width "100%") (cellpadding 0) (cellspacing 0) (border 0))
        ,@(header)
        ,(messages)
        ,(do-body body)
        ,@(footer)))))

  )