;;
;; $Id: form.scm,v 1.11 2005/08/02 19:23:13 ben Exp $
;;
;; Scheme code to be a collection of forms for the sps application.
;;
;; See web/partial-form.scm.
;;

(require-library 'sisc/libs/srfi/srfi-26)
(require-library 'sisc/libs/srfi/srfi-42)
(require-library 'siscweb/bindings)

(require-library 'com/amazingmedia/web/partial-form)
(require-library 'com/amazingmedia/web/form-pred)
(require-library 'com/amazingmedia/util/context)
(require-library 'com/amazingmedia/util/list)
(require-library 'com/amazingmedia/types/shipping-address)
(require-library 'com/amazingmedia/types/credit-card)

(require-library 'com/amazingmedia/sps/web/context-helper)

(module com/amazingmedia/sps/web/form
    (all-forms)

  (import srfi-26)
  (import srfi-42)
  (import com/amazingmedia/util/context)
  (import com/amazingmedia/util/list)
  (import com/amazingmedia/web/partial-form)
  (import com/amazingmedia/web/form-pred)
  (import com/amazingmedia/types/credit-card)
  (import com/amazingmedia/types/shipping-address)
  (import com/amazingmedia/sps/web/context-helper)
  (import siscweb/bindings)
  
  ;;
  ;; Return a context that contains the collection of forms.
  ;;
  (define (all-forms)
    (make-context 
     'immutable
     `((search
        .
        ,(make-partial-form
          '("q")
          (lambda (handler q)
            `(inline-form 
              ,handler
              (prompt "What are you looking for? ")
              (input "q" ,q 20)
              (submit "Search")))
          (list
           (field-filled-in? "q" "query"))
          list))

       (update-cart
        .
        ,(make-partial-form
          '("quantity")
          (lambda (handler q)
            `(inline-form
              ,handler
              (input "quantity" ,q 4)
              (submit "Update")))
          (list
           (field-is-a-positive-number? "quantity" "quantity"))
          (lambda (quantity)
            (list (string->number quantity)))))

       (login
        .
        ,(make-partial-form
          '("username" "password")
          (lambda (handler u p)
            `(form
              ,handler
              (text     "username" "Username: " ,u 20)
              (password "password" "Password: " ,p 20)
              (submit   "Log In")))
          (list 
           (try-all
            (field-filled-in? "username" "username")
            (field-filled-in? "password" "password"))
           (fields-validate? '("username" "password")
                             (lambda (ctx u p)
                               (let-query ctx check-login
                                          (u p) (valid? why)
                                 (if valid? #t 
                                     (string-append "Login failed: username and "
                                                    "password don't match."))))))
          list))
       
       (register
        .
        ,(make-partial-form
          '("username" "password1" "password2")
          (lambda (handler u p1 p2)
            `(form
              ,handler
              (text       "username"    "Username: "         ,u  20)
              (password   "password1"   "Password: "         ,p1 20)
              (password   "password2"   "Confirm Password: " ,p2 20)
              (submit "Register")))
          (list
           (try-all
            (field-filled-in? "username" "username")
            (field-filled-in? "password1" "password")
            (field-filled-in? "password2" "password confirmation"))
           (try-all
            (fields-validate? '("password1" "password2")
                              (lambda (ctx p1 p2)
                                (if (equal? p1 p2) 'ok
                                    "passwords do not match")))
            (fields-validate? '("username")
                              (lambda (ctx u)
                                (let-query ctx username-exists?
                                           (u) (exists?)
                                  (if exists?
                                      "username already exists, try another one"
                                      'ok))))))
          (lambda (u p1 p2) (list u p1))))

       (billing
        .
        ,(make-partial-form
          '("cc-type" "cc-num" "cc-exp-month" "cc-exp-year" 
            "first-name" "last-name" "street1" "street2" "city"
            "state" "zip" "phone" "ship-bill-same")
          (lambda (handler cc a phone ship-same-as-bill?)
            `(form
              ,handler
              (section "Credit Card Information")
              (select     "cc-type" "Credit Card Type:"
                          ("Visa" "Master Card" "American Express")
                          ("visa" "mc" "amex") 
                          ,(and cc (credit-card-type cc)))
              (text       "cc-num" "Card Number:"       
                          ,(and cc (credit-card-number cc)) 18)
              (month-year "cc-exp-" "Expiration Date: "  
                          ,(list-ec (: i 2006 2010) i))
              (section "Billing Address")
              (text    "first-name"       "First Name: "      
                       ,(and a (shipping-address-first-name a)) 20)
              (text    "last-name"        "Last Name: "       
                       ,(and a (shipping-address-last-name a)) 20)
              (text    "street1"          "Street Address: "  
                       ,(and a (shipping-address-street1 a)) 20)
              (text    "street2"          ""                  
                       ,(and a (shipping-address-street2 a)) 20)
              (text    "city"             "City"    
                       ,(and a (shipping-address-city a)) 20)
              (select  "state"            "State" 
                       ("New York" "California" "New Jersey")
                       ("ny" "ca" "nj")
                       ,(and a (shipping-address-state a)))
              (text    "zip"              "Zip"       
                       ,(and a (shipping-address-zip a)) 10)
              (text    "phone"            "Phone"
                       ,phone 15)
              (checkbox "ship-bill-same"  "Ship to is Billing Address?" 
                        ,ship-same-as-bill?)
              (submit  "Continue")))
          (list 
           (apply try-all
                  (map field-filled-in?
                       '("cc-num" "first-name" "last-name" "street1"
                         "city" "state" "zip" "phone")
                       '("credit card number" "first name" "last name" "street 1"
                         "city" "state" "zip" "phone")))
           (fields-validate? '("cc-exp-month" "cc-exp-year")
                             (lambda (ctx month year)
                               (if (cc-expired? (string->number month)
                                                (string->number year))
                                   "credit card is expired"
                                   'ok))))
          (lambda (cc-type cc-num cc-exp-month cc-exp-year first-name last-name
                           street1 street2 city state zip phone ship-bill-same?)
            (list (maybe-make-credit-card cc-num cc-type cc-exp-month cc-exp-year)
                  (make-shipping-address first-name last-name street1 street2
                                         city state zip)
                  phone ship-bill-same?))))

       (shipping
        .
        ,(make-partial-form
          '("first-name" "last-name" "street1" "street2" "city"
            "state" "zip" "phone")
          (lambda (handler a phone)
           `(form
             ,handler
              (text    "first-name"       "First Name: "      
                       ,(and a (shipping-address-first-name a)) 20)
              (text    "last-name"        "Last Name: "       
                       ,(and a (shipping-address-last-name a)) 20)
              (text    "street1"          "Street Address: "  
                       ,(and a (shipping-address-street1 a)) 20)
              (text    "street2"          ""                  
                       ,(and a (shipping-address-street2 a)) 20)
              (text    "city"             "City"    
                       ,(and a (shipping-address-city a)) 20)
              (select  "state"            "State" 
                       ("New York" "California" "New Jersey")
                       ("ny" "ca" "nj")
                       ,(and a (shipping-address-state a)))
              (text    "zip"              "Zip"       
                       ,(and a (shipping-address-zip a)) 10)
              (text    "phone"            "Phone"
                       ,phone 15)
             (submit  "Continue")))
          (list 
           (apply try-all
                  (map field-filled-in?
                       '("first-name" "last-name" "street1"
                         "city" "state" "zip" "phone")
                       '("credit card number" "first name" "last name" "street 1"
                         "city" "state" "zip" "phone"))))
          (lambda (first-name last-name street1 street2 city state zip phone)
            (list (make-shipping-address first-name last-name street1 street2
                                         city state zip)
                  phone))))
       )))
  )