;;
;; $Id: layout.scm,v 1.17 2005/08/08 18:46:41 ben Exp $
;;
;; Scheme code to layout the specifics of this application.  We will register
;; our different domain specific languages and say how they are converted
;; into HTML
;;
;; For example, we have a language for describing a shopping cart, it looks
;; like:
;;   (cart (message "Something was added")
;;         (item (name "Foo") (id "FX139") (price 3.99) (quantity 2))
;;         (item (name "Bar") (id "BX200") (price 2.99) (quantity 10)))
;;
;; Inside of this file, we map this data to HTML presentation.
;;

(require-library 'com/amazingmedia/util/context)
(require-library 'com/amazingmedia/web/domain-ml)
(require-library 'com/amazingmedia/types/credit-card)
(require-library 'com/amazingmedia/types/shipping-address)
(require-library 'com/amazingmedia/sps/web/button)
(require-library 'com/amazingmedia/sps/web/decoration)
(require-library 'sxml/sxml-match)


(module com/amazingmedia/sps/web/layout
    (all-layouts)
  
  (import com/amazingmedia/web/domain-ml)
  (import com/amazingmedia/util/context)
  (import com/amazingmedia/types/credit-card)
  (import com/amazingmedia/types/shipping-address)
  (import sxml-matcher)
  (import com/amazingmedia/sps/web/button)
  (import com/amazingmedia/sps/web/decoration)
  (import srfi-42)
  (import srfi-19)
  ;;
  ;; This procedure returns back a new context with all the 
  ;; system's layouts defined in them.
  ;;
  (define (all-layouts)
    (define ($$ amount)
      (format "$~0,2F" (if amount amount 0)))
    
    ;;
    ;; Our default rule: if this is a symbol or text, return it.
    ;; if it appears to be markup, then complain that we didn't
    ;; find something specific
    ;;
    (make-context
     'immutable
     `((*default*
        .
        ,(lambda (ctx doc)
           (cond ((pair? doc) 
                  (error (format "No specific handler found for: ~a" doc)))
                 (else doc))))

       (landing
        .
        ,(lambda (ctx doc)
           (sxml-match doc
             [(landing (cat ,what ,short-description)
                       ...)
              (decorate-body
               ctx
               `(table (@ (cellpadding 0) (cellspacing 0))
                       (tr
                        (td (@ (class "landingCategories")
                               (valign "top"))
                            (div (@ (class "category"))
                                 ,(button/view-category ctx "" what)
                                 (div (@ (class "description"))
                                      ,short-description)) ...)
                        (td (@ (valign "top") (style "padding: 20px"))
                            (img (@ (src "../images/splash.gif")))))))])))
       
       (simple-results
        .
        ,(lambda (ctx doc)
           (define (fmt content)
             (sxml-match content
               [($ ,amount)    `(td (@ (align "right")) ,($$ amount))]
               [,x (guard (number? x)) `(td (@ (align "right")) ,x)]
               [,x             `(td ,x)]))
           
           (sxml-match doc
             [(simple-results (title ,t)
                              (labels ,label ...)
                              (row ,content ...) ...)
              (decorate-body
               ctx
               `(div (div (@ (class "pageTitle")) ,t)
                     (table (@ (cellpadding 0) (cellspacing 0) (class "queryResults"))
                            (tr (th ,label) ...)
                            (tr
                             ,(fmt content) ...) ...)))])))

       (item-details
        .
        ,(lambda (ctx doc)
           (sxml-match doc
             [(item-details (id ,id) (name ,n) (price ,p) (available ,q) 
                            (description ,d) (image ,i))
              (decorate-body
               ctx
               `(div (@ (class "itemDetails"))
                     (table 
                      (tr
                       (td (@ (rowspan 5)) (img (@ (src-c ,(string-append "/images/" i)))))
                       (td ,n))
                      (tr (td ,($$ p)))
                      (tr (td ,q))
                      (tr (td ,d))
                      (tr (td ,(button/add-to-cart ctx id p 1))))])))))

       (cart
        .
        ,(lambda (ctx doc)
           (sxml-match doc
             [(cart (item (id ,id) (name ,name) (selected ,sel?) (image ,img) 
                          (price ,price) (quantity ,quantity))
                    ...)
              (decorate-body
               ctx
               `(div (@ (class "shoppingCart"))
                     (table (@ (cellpadding 0) (cellspacing 0) (class "queryResults"))
                            (tr (th "Item") (th "Unit Price") (th "Quantity") 
                                (th "Total Price") (th nbsp))
                            (tr (@ (class ,(if sel? "highlighted" #f)))
                                (td (img (@ (src-c ,(string-append "/images/" img))))
                                    nbsp nbsp nbsp
                                    ,name)
                                (td (@ (align "right")) ,($$ price))
                                (td (@ (align "right")) ,(button/update-cart ctx id quantity))
                                (td (@ (align "right")) ,($$ (* price quantity)))
                                (td (@ (align "right"))) ,(button/remove-from-cart ctx id))
                            ...
                            (tr 
                             (td (b "Total:"))
                             (td nbsp) 
                             (td (@ (align "right")) ,(+ quantity ...))
                             (td (@ (align "right")) ,($$ (+ (* price quantity) ...)))
                             (td nbsp)))
                     ,(if (= (length (list id ...)) 0)
                          ""
                          (button/checkout ctx))))))))
       
       (basic-page
        .
        ,(lambda (ctx doc)
           (sxml-match doc
             [(basic-page (title ,title ...)
                          ,content)
              (decorate-body
               ctx
               `(div (div (@ (class "pageTitle")) ,title ...)
                     ,content))])))

       (form
        .
        ,(lambda (ctx doc)
           (sxml-match doc
             [(form ,handler ,[entries] ...)
              `(div (div (@ (class "standardForm"))
                    (form (@ (action-p ,handler)
                             (method "POST"))
                          (table (@ (cellpadding 0) (cellspacing 0))
                                 ,entries ...))))]
             [(section ,title)
              `(tr (td (@ (colspan 2) (class "section")) ,title))]
             [(text ,name ,label ,value ,size)
              `(tr (td ,label)
                   (td (input (@ (type "text") (name ,name) 
                                 (size ,size) (value ,value)))))]
             [(password ,name ,label ,value ,size)
              `(tr (td ,label)
                   (td (input (@ (type "password") (name ,name) 
                                 (value ,value) (size ,size)))))]
             [(select ,name ,label ,display-values ,values ,selected)
              `(tr (td ,label)
                   (td (select (@ (name ,name))
                               ,@(map (lambda (dv v)
                                        `(option (@ (value ,v)
                                                    (selected ,(if (equal? selected v) 
                                                                   "selected" #f)))
                                                 ,dv))
                                      display-values
                                      values))))]
             [(checkbox ,name ,label ,value)
              `(tr (td ,label)
                   (td (input (@ (name ,name) (type "checkbox") (checked ,value)))))]
             [(month-year ,name-prefix ,label ,year-range)
              `(tr (td ,label)
                   (td "Month: "
                       (select (@ (name ,(string-append name-prefix "month")))
                               ,@(list-ec (: i 1 13) `(option ,i)))
                       " "
                       "Year: " (select (@ (name ,(string-append name-prefix "year")))
                                        ,@(list-ec (: y year-range) `(option ,y)))))]
             [(submit ,label)
              `(tr (td (@ (colspan 2))
                       (input (@ (type "submit") (value ,label)))))]
             [,otherwise (error (format "Unknown form element: ~a" otherwise))])))


       (inline-form
        .
        ,(lambda (ctx doc)
           (sxml-match doc
             [(inline-form ,handler ,[entries] ...)
              `(form (@ (action-p ,handler)
                        (method "POST"))
                     ,entries ...)]
             [(prompt ,text) text]
             [(input ,name ,value ,size)
              `(input (@ (type "text") (name ,name) 
                         (value ,value) (size ,size)))]
             [(submit ,label)
              `(input (@ (value ,label) (type "submit")))]
             [,otheriwse (error (format "inline-form: unknown form element: ~a"
                                        otherwise))])))

       
       (main-column-layout
        .
        ,(lambda (ctx doc)
           (sxml-match doc
             [(main-column-layout (title ,title) 
                                  (controls ,control ...)
                                  (section ,s1-title ,[s1-body])
                                  (section ,s2-title ,[s2-body]) ...)
              (decorate-body
               ctx
               `(div (@ (class "multiColumnLayout"))
                     (table (@ (cellpadding 0) (cellspacing 0) (border 0))
                            (tr (td (@ (class "layoutEntry"))
                                    (div (@ (class "title")) ,s1-title)
                                    (div (@ (class "body"))  ,s1-body))
                                (td 
                                 (table 
                                  (@ (cellpadding 0) (cellspacing 0))
                                  (tr (td (@ (class "layoutEntry"))
                                          (div (@ (class "title")) ,s2-title)
                                          (div (@ (class "body")) ,s2-body)))
                                  ...)))
                            (tr (td (@ (colspan 2))
                                    (span ,control nbsp nbsp nbsp) ...)))))]
             [(cart (item (id ,id) (name ,name) (selected ,_) 
                          (image ,image) (price ,price) 
                          (quantity ,quantity)) ...)
              `(table (@ (class "purchaseInfo"))
                      (tr (td ,id) (td ,name) (td ,($$ price)) (td ,quantity)
                          (td ,($$ (* price quantity)))) ...)]
             [(info-block ,v) (guard (credit-card? v))
              `(table (@ (class "infoBlock"))
                      (tr (th "Credit Card Number:") (td ,(credit-card-number v))) 
                      (tr (th "Credit Card Type:") (td ,(format-cc-type
                                                         (credit-card-type v))))
                      (tr (th "Credit Card Expiration:") 
                          (td ,(date->string (credit-card-expires v) "~m/~Y"))))]
             [(info-block ,v) (guard (shipping-address? v))
              `(table (@ (class "infoBlock"))
                      (tr (th "To:") (td ,(shipping-address-first-name v) " "
                                         ,(shipping-address-last-name v)))
                      (tr (th "Street:") (td ,(shipping-address-street1 v)))
                      ,(if (equal? "" (shipping-address-street2 v)) ""
                           `(tr (th "")        (td ,(shipping-address-street2 v))))
                      (tr (th "City:") (td ,(shipping-address-city v)))
                      (tr (th "State:") (td ,(format-state 
                                              (shipping-address-state v))))
                      (tr (th "Zip:") (td ,(shipping-address-zip v))))]
             [(info-block ,v) (error (format "Don't know how to render an info block: ~a"
                                             v))]
             [,otherwise (error (format "main-column-layout: unknown element: ~a" 
                                        otherwise))])))

       (orders
        .
        ,(lambda (ctx doc)
            (sxml-match doc
              [(orders (order ,order-id ,ship-to 
                               ,total ,purchase-date
                               (line ,line-number ,item-id ,item-name
                                     ,quantity ,price) ...) ...)
               (decorate-body 
                ctx
                `(div
                  (div (@ (class "pageTitle")) "Account History")
                  (div (@ (class "orders"))
                       (table 
                        (tr (@ (class "order"))
                            (td (@ (valign "top") (class "summary"))
                                (table (tr (td "Order ID:") (td ,order-id))
                                       (tr (td "Ordered:") 
                                           (td ,(date->string (time-monotonic->date purchase-date) "~D")))
                                       (tr (td "Sent To:")  (td ,ship-to))
                                       (tr (td "Total: ")   (td ,($$ total)))))
                            (td (@ (valign "top") (class "details"))
                                (table 
                                 (tr (th "")
                                     (th "Item Purchased")
                                     (th "Quantity")
                                     (th "Price"))
                                 (tr 
                                  (td ,line-number)
                                  (td ,item-id " ",item-name)
                                  (td ,quantity)
                                  (td ,($$ price)))
                                 ...)))
                        ...))))])))
        
        )))
    )