;;
;; $Id: nav-points.scm,v 1.9 2005/08/08 18:47:11 ben Exp $
;;
;; A nav point is a top level function that can be invoked
;; to produce a page in the system. It's nice to be able to
;; referer these points in the program in a symbolic way, such
;; as 'view-item and have that map to 
;;    com/amazingmedia/sps/ui/view -> view/item
;;
;; a nav point follows the contract that it must have a first
;; argument which is a context (the state of the system) and
;; can have any other arguments as well.
;;

(require-library 'com/amazingmedia/sps/ui/view)
(require-library 'com/amazingmedia/sps/ui/landing)
(require-library 'com/amazingmedia/sps/ui/search)
(require-library 'com/amazingmedia/sps/ui/cart)
(require-library 'com/amazingmedia/sps/ui/login)
(require-library 'com/amazingmedia/sps/ui/register)
(require-library 'com/amazingmedia/sps/ui/account)
(require-library 'com/amazingmedia/sps/ui/checkout)
(require-library 'com/amazingmedia/util/context)
(require-library 'com/amazingmedia/web/messages)

(module com/amazingmedia/sps/web/nav-points
    (all-nav-points)

  (import com/amazingmedia/util/context)
  (import com/amazingmedia/web/messages)

  (import com/amazingmedia/sps/ui/view)
  (import com/amazingmedia/sps/ui/landing)
  (import com/amazingmedia/sps/ui/search)
  (import com/amazingmedia/sps/ui/cart)
  (import com/amazingmedia/sps/ui/login)
  (import com/amazingmedia/sps/ui/register)
  (import com/amazingmedia/sps/ui/account)
  (import com/amazingmedia/sps/ui/checkout)


  (define (all-nav-points)
    (define (! fn)
      (lambda (ctx . args)
        (apply fn (cons ctx  args))))


    (make-context 
     'immutable
     `((landing          . ,(! landing))
       (view-category    . ,(! view/category))
       (view-product     . ,(! view/product))
       (view-item        . ,(! view/item))
       (search           . ,(! search))
       (add-to-cart      . ,(! cart/add))
       (remove-from-cart . ,(! cart/remove))
       (update-cart      . ,(! cart/update))
       (view-cart        . ,(! cart/view))
       (checkout         . ,(! checkout/start))
       (view-account     . ,(! account/view))
       (login            . ,(! login/prompt))
       (logout           . ,(! login/logout))
       (register         . ,(! register/prompt)))))

  )