;;
;; $Id: credit-card.scm,v 1.3 2005/08/01 15:24:16 ben Exp $
;;
;; Scheme code to encapsulate all of the data associated
;; with a credit card
;;

(require-library 'sisc/libs/srfi/srfi-19)
(require-library 'sisc/libs/srfi/srfi-16)

(module com/amazingmedia/types/credit-card
    (make-credit-card
     maybe-make-credit-card
     credit-card-number
     credit-card-type
     credit-card-expires
     credit-card?
     cc-expired?
     format-cc-type)

  (import record)
  (import srfi-19)
  (import srfi-16)

  (define-struct credit-card
    (number type expires))

  ;;
  ;; Is this credit card expired?
  ;;
  (define cc-expired? 
    (case-lambda 
     ((cc)
      (let ((now (current-date)))
        (time< (date->time-utc (credit-card-expires cc))
               (date->time-utc now))))
     ((month year)
      (let ((now (current-date))
            (maybe (make-date 0 0 0 0 1 month year 0)))
        (time<? (date->time-utc maybe)
               (date->time-utc now))))))

  ;;
  ;; Make a credit-card object, but don't assume we are given valid
  ;; values.
  ;;
  ;; If any of the values are invalid, just return #f into the 
  ;;  (make-credit-card ...) structure.
  ;;
  ;; return a possibly #f filled credit card
  ;;
  (define (maybe-make-credit-card cc-num cc-type cc-exp-month cc-exp-year)
    (make-credit-card cc-num
                      (with/fc (lambda (e c) #f)
                               (lambda ()
                                 (string->cc-type cc-type)))
                      (with/fc (lambda (e c) #f)
                               (lambda ()
                                 (make-date 0 0 0 0 1
                                            (string->number cc-exp-month)
                                            (string->number cc-exp-year)
                                            0)))))

  (define (format-cc-type cc-type)
    (case cc-type
      ((visa) "Visa")
      ((amex) "American Express")
      ((mc) "Master Card")
      (else (error (format "Unkown cc type: ~a" cc-type)))))

  (define (string->cc-type cc-type)
    (case (string->symbol cc-type)
      ((visa amex mc) (string->symbol cc-type))
      (else (error (format "Unknown cc type: ~a" cc-type)))))
  )