;;
;; $Id: list.scm,v 1.3 2005/08/02 19:23:13 ben Exp $
;;
;; Scheme code to provide additional list functions.
;; This package builds on srfi-1 adding some additional functions
;; that I found missing
;;

(require-library 'sisc/libs/srfi/srfi-1)

(module com/amazingmedia/util/list
    (alist-cons-replace
     let-alist
     set-alist!)

  (import srfi-1)

  ;;
  ;; We do an (alist-cons ...) but, if the key
  ;; is found in our existing alist, then remove it.
  ;;
  ;; This allows you to add a new pair to your alist
  ;; and have it replace the existing value for the
  ;; key/value pair.
  ;;
  ;; key -- to store in the alist
  ;; datum -- the data to store
  ;; alist -- the alist
  ;; return an update alist
  ;;
  (define (alist-cons-replace key datum alist)
    (alist-cons key datum
                (alist-delete key alist)))

  ;;
  ;; Just like a (let  ...) but pull bindings from
  ;; an alist.
  ;;
  (define-syntax let-alist 
    (syntax-rules ()
      ((_ ((var key) ...) alist body ...)
       (let* ((var (cond ((assoc key alist) => cdr) (else #f)))
              ...)
         body ...))))

  ;;
  ;; Using mutation, set the value of a key in the alist. Adding
  ;; it into the list if necessary.
  ;;
  ;; key   -- the key to set
  ;; value -- the new value, we will set
  ;; alist -- the list to set
  ;; returns an undefined value. use for side effect.
  ;;
  (define (set-alist! alist key value)
    (let loop ((alist alist))
      (cond ((null? alist) alist)
            ((equal? (caar alist) key)
             (set-cdr! (car alist) value))
            ((null? (cdr alist))
             (set-cdr! alist (cons (cons key value) '())))
            (else (loop (cdr alist))))))

  )