;;
;; $Id: domain-ml.scm,v 1.5 2005/07/28 19:59:56 ben Exp $
;;
;; Scheme code to represent domain-specific markup languages.
;; The idea is this: you write your web functions to spit out
;; some SXML compliant markup language in *your* domain, and then
;; they get transformed automagically into HTML.
;;
;; This package doesn't really invent anything new. It just provides
;; a way of registering style sheet functions and invoking them.
;;
;; This package assumes that your style sheets will make use of
;; sxml-match or some such magic.
;;
;; Also, this package assumes you are using siscweb to actually
;; send the output to a browser, though this would be easy to 
;; parameterize
;; 

(require-library 'siscweb/html)
(require-library 'sisc/libs/srfi/srfi-1)
(require-library 'com/amazingmedia/util/context)

(module com/amazingmedia/web/domain-ml
    (domain-ml/transform
     domain-ml/send)

  (import srfi-1)
  (import siscweb/html)
  (import com/amazingmedia/util/context)
  
  ;;
  ;; Execute the transformation of our domain-ml into
  ;; HTML (or whatever the end language is).
  ;;
  ;; We assume that someone has registered a handler
  ;; for this type.
  ;;
  ;; The special handler *default* will be invoked if nothing
  ;; matches, but a *default* is provided
  ;;
  ;; ctx -- the context to pass around through our web app
  ;; key -- the key to use to look up our transforms in the context
  ;; doc -- the document to transform 
  ;; return the transformed document
  ;;
  (define (domain-ml/transform ctx key doc)
    (cond ((context/get (if (pair? doc) (car doc) doc)
                        (context/get key ctx)) =>
                        (lambda (handler)
                          (handler ctx doc)))
          ((context/get '*default* (context/get key ctx)) =>
           (lambda (handler)
             (handler ctx doc)))
          (else (error (format "No handler defined for the document: ~a" doc)))))

  ;;
  ;; As a convenience this function takes in a document and sends
  ;; it to the users browser using siscweb specific code.
  ;;
  ;; ctx    -- the context passed through our app 
  ;; key    -- the key to look up the transform context
  ;; doc    -- the document to send to the browser
  ;; final? -- says whether or not this should terminate 
  ;;           the user's continuation's. This is useful for 
  ;;           final steps, such as when a user logs out. You
  ;;           want all of their previous continuations to go away.
  ;; does not return
  ;;
  (define (domain-ml/send ctx key doc final?)
    (let ((processed (domain-ml/transform ctx key doc)))
      (if (console-mode)
          (begin 
            (pretty-print processed)
            (newline))
          ((if final? send-html/finish send-html/back) processed))))
  
  )