;;
;; $Id: history.scm,v 1.2 2005/08/02 19:23:13 ben Exp $
;;
;; Scheme code to represent a user's history in a web application.
;; This is made up of both a label as well as procedure to
;; execute to cause the historical action to happen again.
;;
;; By keeping a list of history, we can give the user a fairly cool
;; navigational aid.
;;
;; See com/amazingmedia/web/item for a description of a module
;; very much like this one.
;;
(require-library 'sisc/libs/srfi/srfi-9)
(require-library 'sisc/libs/srfi/srfi-19)
(require-library 'com/amazingmedia/util/context)

(module com/amazingmedia/web/history
    (make-history
     history-label
     history-proc
     history<?
     combine-history
     make-history-context)

  (import srfi-9)
  (import srfi-19)
  (import com/amazingmedia/util/context)

  (define-record-type history
    (make-history-private label proc created)
    history?
    (label history-label set-history-label!)
    (proc history-proc set-history-proc!)
    (created history-created set-history-created!))

  (define (make-history label proc)
    (make-history-private label proc (current-time)))

  (define (history<? h1 h2)
    (time<? (history-created h2)
            (history-created h1)))

  ;;
  ;; When joining history entries, throw away the
  ;; old entry and keep the new one
  ;;
  (define (combine-history h1 h2)
    h2)
  
  ;;
  ;; Make us a context designed to hold history
  ;;
  (define (make-history-context)
    (make-context 'mutable '() history<? equal? combine-history))

)