;;
;; $Id: item.scm,v 1.3 2005/08/02 19:23:13 ben Exp $
;;
;; Scheme code to model an item you can buy from a store.
;; These things are often dropped into, surprise, a shopping
;; cart.
;;

(require-library 'sisc/libs/srfi/srfi-9)
(require-library 'sisc/libs/srfi/srfi-19)
(require-library 'com/amazingmedia/util/context)

(module com/amazingmedia/web/item
    (make-item
     item-id
     item-created
     item-price
     item-quantity
     item?
     combine-items
     item<?
     make-item-context
     update-item-quantity)

  (import srfi-9)
  (import srfi-19)
  (import com/amazingmedia/util/context)

  (define-record-type item 
    (make-item-private id price quantity created)
    item?
    (id item-id set-item-id!) 
    (created item-created set-created!) 
    (price item-price set-price!) 
    (quantity item-quantity set-quantity!))

  (define (make-item id price quantity)
    (make-item-private id price quantity 
                       (current-time)))


  ;;
  ;; Given an existing item, update it to have the new
  ;; quantity
  ;;
  (define (update-item-quantity old-item quantity)
    (make-item-private (item-id old-item)
                       (item-price old-item)
                       quantity
                       (item-created old-item)))


  ;;
  ;; define a way to combine two items together. This is useful
  ;; when we add something to a shopping cart, then try adding
  ;; the item again.
  ;;
  ;; We don't want two items, just the same item, but with
  ;; more quantity
  ;;
  ;; Note: we keep the create as the same as the old item, which 
  ;; is useful because the user doesn't want to think of this
  ;; as a new item.
  ;;
  ;; old-item -- the old item
  ;; new-item -- the new item, the same as the old item, but maybe
  ;;             a different quantity
  ;;
  (define (combine-items old-item new-item)
    (make-item-private (item-id old-item)
                       (item-price old-item)
                       (+ (item-quantity old-item)
                          (item-quantity new-item))
                       (item-created old-item)))
                          
    
  ;;
  ;; compare two different items. This is useful for ordering
  ;; them in a list.
  ;;
  ;; take in two items and return true if the first
  ;; is less than the second.
  ;;
  (define (item<? item1 item2)
    (time>=? (item-created item1)
             (item-created item2)))


  ;;
  ;; As a convenience, provide a way of creating a context object
  ;; which is designed to hold just items.
  ;;
  ;; return an empty context
  ;;
  (define (make-item-context)
    (make-context 'mutable '() item<? equal? combine-items))

  )