-- -*- mode: sql -*-
--
-- $Id: schema.sql,v 1.8 2005/07/28 05:03:10 ben Exp $
--
-- The Schema for the Java Pet Store. Re-engineered from
-- the src files provided by Sun. 
--
-- See: petstore1.3.2/src/apps/petstore/src/docroot/populate/
-- for the data data and scheme used in the app
--

CREATE DOMAIN textId        AS varchar(10);
CREATE DOMAIN id            AS bigint;


CREATE TABLE Category (
    categoryId   textId         PRIMARY KEY,
    name         varchar(80)    NOT NULL,
    image        varchar(255)   NOT NULL
);

CREATE TABLE Product (
    productId    textId         PRIMARY KEY,
    categoryId   textId         REFERENCES Category,
    name         varchar(80)    NOT NULL,
    image        varchar(255)   NULL,
    description  varchar(255)   NULL 
);

CREATE TABLE Item (
    itemId       textId         PRIMARY KEY,
    productId    textId         REFERENCES Product,
    listPrice    numeric(20,2)  NOT NULL,
    unitCost     numeric(20,2)  NOT NULL,
    image        varchar(255)   NOT NULL,
    description  varchar(255)   NOT NULL,
    attr1        varchar(80)    NULL,
    attr2        varchar(80)    NULL,
    attr3        varchar(80)    NULL,
    attr4        varchar(80)    NULL,
    attr5        varchar(80)    NULL
);

CREATE TABLE Inventory (
    itemId      textId          REFERENCES Item,
    quantity    integer         NOT NULL
);


CREATE SEQUENCE userIdSeq;
CREATE TABLE Users (
    userId      id              PRIMARY KEY,
    username    varchar(30)     UNIQUE,
    password    varchar(30)
);

CREATE SEQUENCE addressIdSeq;
CREATE TABLE Address (
    addressId   id              PRIMARY KEY,
    firstName   varchar(30)     NOT NULL,
    lastName    varchar(30)     NOT NULL,
    street1     varchar(60)     NOT NULL,
    street2     varchar(60)             ,
    city        varchar(60)     NOT NULL,
    state       char(2)         NOT NULL,
    zip         varchar(10)     NOT NULL
);


CREATE SEQUENCE ccIdSeq;
CREATE TABLE CreditCard (
    ccId        id              PRIMARY KEY,
    ccNum       varchar(20)     NOT NULL,
    ccType      varchar(20)     NOT NULL,
    ccExpMonth  integer         NOT NULL,
    ccExpYear   integer         NOT NULL
);


CREATE SEQUENCE orderIdSeq;
CREATE TABLE Orders (
    orderId     id              PRIMARY KEY,
    userId      id              REFERENCES Users,
    orderDate   timestamp       NOT NULL,
    ccId        id              REFERENCES CreditCard,
    billAddrId  id              REFERENCES Address(addressId),
    shipAddrId  id              REFERENCES Address(addressId),
    status      varchar(20)     NOT NULL
);

CREATE TABLE OrderLineItem (
    orderId     id              NOT NULL,
    lineNum     integer         NOT NULL,
    itemId      textId          REFERENCES Item,
    quantity    integer         NOT NULL,
    unitCost    numeric(20,2)   NOT NULL
);
